//----------------------------------------------------------------------------------
// File:   GenerateVSM.fx
// Author: Kevin Myers
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#include "VarianceShadowMapping.fxh"


struct VSIN
{
    float3 pos : SV_Position;
    float3 norm : NORMAL;
};

struct PSIN
{
    float4 pos : SV_Position;
    float fDepth : fDepth;
};

PSIN VSGenerateVSM( VSIN input )
{
    PSIN output = (PSIN)0.0f;
    
    float4 worldPos = mul( float4( input.pos, 1.0f ), mWorld );
    
    output.pos = mul( mul( worldPos, mShadowView ), mShadowProj );
    output.fDepth = length( vLightPos.xyz - worldPos.xyz );

    return output;
}

float4 PSGenerateVSM( PSIN input ) : SV_Target
{
    return float4( input.fDepth, input.fDepth * input.fDepth, 0.0f, 1.0f );
}

technique10
{
    pass p0
    {
        SetVertexShader( CompileShader( vs_4_0, VSGenerateVSM() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PSGenerateVSM() ) );
        
        SetRasterizerState( noCullRS );
        SetBlendState( RGBABS, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xffffffff );
        SetDepthStencilState( zWriteNoStencilDS, 0xffffffff );
    }
}