//----------------------------------------------------------------------------------
// File:   SeparableBlur.fx
// Author: Kevin Myers
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#include "VarianceShadowMapping.fxh"

bool bVertical = false;
float fStepSize = 1.0f;
Texture2D blurTexture;

float4 PSFilter( VSQuadOut input ) : SV_Target
{
    //PreShader - This should all be optimized away by the compiler
    //====================================
    float fStartOffset = BoxFilterStart( fFilterWidth );
    float2 fTexelOffset = float2( bVertical * ( fStepSize / fTextureWidth ), !bVertical * ( fStepSize / fTextureWidth ) );
    //====================================
    
    float2 fTexStart = input.tex - ( fStartOffset * fTexelOffset );
    float4 output = (float4)0.0f;
    
    for( int i = 0; i < fFilterWidth; ++i )
        output += blurTexture.Sample( FILT_LINEAR, float2( fTexStart + fTexelOffset * i) );
    
    return output / fFilterWidth;
}


technique10
{
    pass p0
    {
        SetVertexShader( CompileShader( vs_4_0, VSFullscreenQuad() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PSFilter() ) );
    
        SetRasterizerState( noCullRS );
        SetBlendState( RGBABS, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xffffffff );
        SetDepthStencilState( zWriteNoStencilDS, 0xffffffff );
    }
}